make.package <-
function(name, functionS, packageData=NULL,
                         descriptionFile=NULL,
                         namespaceFile=NULL, vignetteFiles=NULL,
                         manPath=NULL, citationFile=NULL,
                         newsFile=NULL, doClean=TRUE, doInstall=TRUE,
                         doCheck=FALSE, doVignettes=TRUE, ...){
    ## name: A character determining the name of the package.
    ## functionS: A character vector naming the R functions to put in the package. The
    ## corresponding scripts that define these functions should have been already sourced
    ## before running make.package().
    ## packageData: A character vector. Each element is the name of an object to
    ## be put in the package.
    ## descriptionFile: The/path/to/the/DESCRIPTION file.
    ## namespaceFile: The/path/to/the/NAMESPACE file.
    ## doVignettes: If FALSE, vignettes will not be built e.g., For faster test
    ## build or timing according to the bioconductor guideline. 
    ## ...: passed to package.skeleton(...).
    ## History: Habil converted Shiva's ~/proj/dementia/code/senescence/Shiva/packing.R
    ## script to this function to facilitate making R packages.
    ##2021-04-28.

    result <- list()
    print(paste( "Making the", name, "package..."))
    result[["name"]] <- name

    ## Backup:
    if(dir.exists(name)){
        dir.create("backup", showWarnings=FALSE)
        file.copy(from=name, to= "backup", recursive=TRUE, overwrite=TRUE)
    }
    unlink(name, recursive=TRUE)
    
    ## create the skeleton
    ## QC for functionS:
    if(all(functionS %in% ls()))
        stop("All functions are not available in Global environment!")
    ## QC for functionS:
    if(all(packageData %in% ls()))
        stop("All data are not available in Global environment!")
    package.skeleton(name=name, list=c(functionS, packageData), ...)
    ## Remove manual templates
    unlink(file.path(name, "man"), recursive=TRUE)
    if(!is.null(manPath)){
        file.copy(from=manPath, to=name, overwrite=TRUE, recursive=TRUE)
    }

    ## Copy the description
    if(!is.null(descriptionFile)){
        file.copy(from=descriptionFile, to=file.path(name, "DESCRIPTION"), overwrite=TRUE)
    }
    ## Copy the name space
    if(!is.null(namespaceFile)){
        file.copy(from=namespaceFile, to=file.path(name, "NAMESPACE"), overwrite=TRUE)
    } else {
        writeLines(con=file.path(name, "NAMESPACE"), text='exportPattern("^[[:alpha:]]+")')
    }
    ## Copy the Vignettes
    if(!is.null(vignetteFiles)){
        vignettePath <- file.path(name, "vignettes")
        dir.create(vignettePath, recursive=FALSE, showWarnings=TRUE)
        file.copy(from=vignetteFiles, to=vignettePath, overwrite=TRUE)
    }
    ## Copy NEWS
    if(!is.null(newsFile)){
        file.copy(from=newsFile, to=name, overwrite=TRUE)
    }
    ## Copy CITATION
    if(!is.null(citationFile)){
        instPath <- file.path(name, "inst")
        dir.create(instPath, recursive=FALSE, showWarnings=TRUE)
        file.copy(from=citationFile, to=instPath, overwrite=TRUE)
    }

    version1 <-
        system(paste0("sed -n 's/^Version: *//p' ", name, "/DESCRIPTION"),
               intern=TRUE)
    result[["packageVersion"]] <- version1

    ##Vignettes or not?
    if(!doVignettes){
        vignettesFlag <- "--no-build-vignettes"
    } else {
        vignettesFlag <- ""
    }
    vignettesFlag <- paste(vignettesFlag, "--resave-data=best")
    print(paste("vignettesFlag: ", vignettesFlag))

    print(paste("Building", name, "version:", version1))
    print(Sys.time())
    system(paste("R CMD build", vignettesFlag, name))
    tarball <- paste0(name,"_", version1,".tar.gz")
    print(paste("The tarball is:", tarball))

    ## Check:
    print(paste("doCheck:", doCheck))
    if(doCheck){
        print(Sys.time())
        system(paste("R CMD check", vignettesFlag, tarball))
    }

    ## Install:
    if(doInstall){
        if(require(name)){
            remove.packages(pkgs=name)
        }
        install.packages(pkgs=tarball)
    }

    ## Cleaning
    if(doClean)
        unlink(name, recursive=TRUE)
    
    ## End time:
    print(paste("Packing", name, "finished at:"))
    print(Sys.time())
    return(result)
}
