surv.models <-
function(depVar, timeVar, outData, eigSet, covars, saveAllCoefs=TRUE, label=NULL,
                        weed=FALSE, verbose=1, modelType="log-rank", outPath=experimentPath,
                        txtFile="Auto", ## Set to NULL to disable.
                        doKmWithNa=FALSE){
    ## Meghan made this its own function on 2020-4-28. It takes in a dataset of eigenproteins
    ##(i.e., the eigSet matrix) and an outcome dataset (i.e., the outData data frame),
    ## as well as strings representing the dependent censoring variable
    ##(i.e., depVar, e.g. 1 for dementia and 0 otherwise)
    ## and time to event variable (i.e., timeVar, e.g., years from blood draw till dementia onset),
    ## and vector of covariate names.
    ## It fits Cox PH models, returns and saves their coefficients in csv files,
    ## and plots KM curves.

    result <- list()
    ## Make a directory to store results
    depVarPath <- file.path(outPath, depVar)
    dir.create(depVarPath, recursive=TRUE, showWarnings=FALSE)
    if(txtFile=="Auto")
        txtFile <- file.path(depVarPath, "surv.models.txt")
    printf <- function(...){
        return(message.if(verbose=verbose, txtFile=txtFile, ...))
    }
    if(file.exists(txtFile))
        file.remove(txtFile)
    printf("surv.models()...")
    ##QC:
    if(is.null(rownames(outData)))
        stop("outData must have row names!")
    ## shareid
    if(! "shareid" %in% colnames(outData)){
        outData <- cbind(outData, "shareid"= rownames(outData))
        warning("outData did not have the shareid column. Row names was used in lieu of shareid.")
    }

    ## merge mri and protein tot
    tot <- merge(eigSet, outData, by=0)
    result[["Before_excl"]] <- nrow(tot) 
    eignames <- colnames(eigSet)
    ## remove people with missing depVar
    narroWithNa <- tot[, c("shareid", depVar, timeVar, eignames, covars) ]  
    result[["missingTimeVar"]] <- sum(is.na(tot[,timeVar]))
    result[["missingDepVar"]] <- sum(is.na(tot[,depVar]))
    ## remove people with missing vars
    narrow <- na.omit(narroWithNa)
    printf(paste("N before removing NAs:"))
    printf(nrow(narroWithNa))
    printf("Number of NAs...")
    message.if(colSums(is.na(narroWithNa)), txtFile=txtFile, verbose=0)
    printf(paste("N in", depVar, "analysis:"))
    printf(nrow(narrow))
    result[["Excl_missing"]] <- nrow(narrow)

    ph.dummy <- function(f1){
        if(!inherits(f1,"formula"))
            stop("f1 must inherit from call formula!")
        model <- coxph(f1, data=narrow)
        return(model)
    }

    ####
    ##module2coefs <- list()
    ####
    
    weeded <- list()
    weedingPath <- file.path(depVarPath, "weeding")
    dir.create(weedingPath, recursive=TRUE, showWarnings=FALSE)

    ####
    coefS <- NULL
    ####

    ## Per eigenprotein:
    narroWithNa <- cbind(narroWithNa, "zcol1"=NA)
    narrow <- cbind(narrow, "zcol1"=NA)
    for(col1 in colnames(eigSet)){
        printf(col1)
        column <- narrow[, col1]
        narrow[, "zcol1"] <- (column - mean(column))/sd(column)
        ##browser()

        ## Violin plot
        viData <- narroWithNa[,c(depVar, col1)]
        class(viData[,depVar]) <- "character"
        gg1 <- ggplot(viData, aes(x=get(depVar), y=get(col1), fill=get(depVar)))
        gg1 <- gg1 + geom_violin(position="dodge", alpha=0.5)
        gg1 <- gg1 + labs(x=depVar, y=col1, fill=depVar)
        counts1 <- as.data.frame(table(viData[,depVar]))
        ##gg1 <- gg1 + geom_text(data=counts1 ,aes(x=Var1, y=0.1, label=Freq),
        ##                     color="red", fontface =2, size = 5)
        ##^Habil: Adding counts did not work,
        ## https://stackoverflow.com/a/65254557
        ## Add numbers to the legend:
        nums <- table(viData[,depVar],useNA="ifany")
        nums <- setNames(nums, nm=paste(names(nums)))
        gg1 <- gg1 + scale_fill_discrete(labels=paste0(names(nums), " (n=", nums,")"))
        gg1
        ggsave(file.path(depVarPath, paste0( "violin_", col1, ".png")))

        if(is.null(covars)){
            f0 <- paste0("Surv(", timeVar, ",", depVar, ") ~ ", "zcol1")
        } else {
            f0 <- paste("Surv(", timeVar, ",", depVar, ") ~ ",
                        "zcol1", " + ", paste(covars, collapse = " + "))
        }
        f0 <- as.formula(f0)
        lmtemp <-ph.dummy(f1=f0)
        phCheck <- cox.zph(lmtemp)
        write.csv(phCheck$table, file = file.path(outPath, depVar,
                                          paste("phCheck_", depVar, col1, ".csv", sep = "")))
        coefs <- summary(lmtemp)$coefficients
        rownames(coefs)[which(rownames(coefs)=="zcol1")] <- col1

        ####
        ##module2coefs[[col1]] <- coefs
        ####


        ####
        if(is.null(coefS)){
            rows <- rownames(coefs)
            rows[grep(rows, pattern="ME")] <- "ME"
            coefS <- array(, dim=c(nrow(coefs), ncol(coefs), ncol(eigSet)),
                           dimnames=list(rows, colnames(coefs), colnames(eigSet)))
        }     
        coefS[,,col1] <- coefs
        ##browser()
        
        ####
        
        if(saveAllCoefs==TRUE){
            write.csv(coefs, file = file.path(outPath, depVar,
                                              paste("coefs_", depVar, col1, ".csv", sep = "")))
        }
        if(weed==TRUE){
            printf(paste0("Pruning module ", col1, " for ", depVar, " outcome..."))
            if(!is.null(pigengeneFile)){
                weeded[[col1]] <- weed.proteins.surv(pigengeneFile=pigengeneFile,
                                                     depVar=depVar, timeVar=timeVar,
                                                     indepVar=col1, outData=outData,
                                                     covars=covars,
                                                     verbose=verbose-1,
                                                     outPath=file.path(weedingPath, col1))
            }
        }
        
        ## Add KM curves for quantiles:
        if(! col1 %in% c("ME2", "ME4")){
            next
        }
        if(!doKmWithNa){
            kmData <- narrow
        } else {
            columnWithNa <- narroWithNa[,col1]
            zcor1Na <- (columnWithNa - mean(columnWithNa))/sd(columnWithNa)
            narroWithNa[, "zcol1"] <- zcor1Na
            kmData <- narroWithNa
        }
        protVec <- kmData$zcol1
        thresholds <- km.cutoffs(dataSet=kmData, modName=col1, timeVar=timeVar, depVar=depVar,
                                 modelType=modelType, covars=covars)
        result[[paste0(col1, "_thresholds")]] <- thresholds
        bestThresh <- thresholds$bestThresh
        kmData$eigcat <- NA
        kmData[kmData$zcol1 <   quantile(protVec, bestThresh), "eigcat"]  <- "Low"
        kmData[kmData$zcol1 >=  quantile(protVec, bestThresh), "eigcat"]  <- "High"
        ## KM:
        form1 <- as.formula(paste0("Surv(", timeVar, ",", depVar, ") ~ eigcat"))
        km <- do.call(survfit, args = list(formula = form1, data = kmData))
        if(FALSE){ ## Not needed, Habil.
            if(is.null(covars)){
                f2 <- paste0("Surv(", timeVar, ",", depVar, ") ~ ", "eigcat")
            } else {
                f2 <- paste("Surv(", timeVar, ",", depVar, ") ~ ",
                            "eigcat", " + ", paste(covars, collapse = " + ") )
            }
            f2 <- as.formula(f2)
            bestPH <- ph.dummy(f1=f2)
        }
        kmplotFile <- file.path(depVarPath, paste0(col1, "_", depVar, "best.png"))
        num <- table(kmData[,"eigcat"])
        hiLoLabs <- paste0(rownames(num), " ", col1, " (n=", num, ")")
        kmplot <- ggsurvplot(km, data = kmData,
                             pval = paste0("Adjusted p-value: p=",
                                           round(thresholds$best["pVal"],3)),
                             legend.labs=hiLoLabs)
        kmplot
        ggsave(kmplotFile, dpi=600)
        printf(paste(col1, "KM plot was saved in:", kmplotFile))
        ## km plotted.     
    } ## col1 (eigenprotein)

    ## Output:
    ####
    ##result[["coefs"]] <- module2coefs
    ####
    result[["coefS"]] <- coefS
    result[["samplemod"]] <- lmtemp
    result[["weeded"]] <- weeded
    models <- result
    save(models, file=file.path(depVarPath, paste(depVar, label, "models.RData", sep="_")))
    save(coefS, file=file.path(depVarPath, paste(depVar, label, "coefS.RData", sep="_")))
    cat("\n\n Survival results saved at:\n")
    cat(file.path(outPath, depVar), "\n")
    return(result)
}
